﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace DAMICreator {
    public partial class DAMIForm : Form {
        private byte[] darkages;
        private List<uint> miOffsets = new List<uint>();

        public DAMIForm() {
            InitializeComponent();
            this.Text += " v" + Application.ProductVersion;
            openFileDialogDarkages.InitialDirectory = saveFileDialogDarkagesMI.InitialDirectory = Application.StartupPath;
        }

        private void buttonOpenDarkages_Click(object sender, EventArgs e) {
            byte totalSearchBytes;
            string[] miSearchValueTemp = textBoxDAMISearch.Text.Split(new string[] { "-" }, StringSplitOptions.RemoveEmptyEntries);
            if(!byte.TryParse(textBoxSearchBytes.Text, out totalSearchBytes) || totalSearchBytes > miSearchValueTemp.Length || totalSearchBytes == 0) { MessageBox.Show("Make Sure the Total Search Byte is Valid"); return; }

            //string miSearchValue = textBoxDAMISearch.Text.Substring(textBoxDAMISearch.Text.Length - totalSearchBytes * 3 + 1, totalSearchBytes * 3 - 1); //Search Using Regular Expression Method
            byte[] miSearchValue = new byte[totalSearchBytes];
            for(byte i = (byte)(miSearchValueTemp.Length - totalSearchBytes); i < miSearchValueTemp.Length; i++)
                if(!byte.TryParse(miSearchValueTemp[i], NumberStyles.HexNumber, CultureInfo.InvariantCulture, out miSearchValue[i - miSearchValueTemp.Length + totalSearchBytes])) { MessageBox.Show("Make Sure the Search String is Valid"); return; }

            if(openFileDialogDarkages.ShowDialog(this) == DialogResult.OK) {
                miOffsets.Clear();
                //groupBoxResults.Enabled = FindSetMIValuesRegEx(openFileDialogDarkages.FileName, miSearchValue); //Search Using Regular Expression Method
                groupBoxResults.Enabled = FindSetMIValuesBinary(openFileDialogDarkages.FileName, miSearchValue);

                listBoxMatches.DataSource = null;
                if(groupBoxResults.Enabled) listBoxMatches.DataSource = miOffsets;
                else MessageBox.Show("No MI Matches Found");
            }
        }

        private bool FindSetMIValuesBinary(string daExcutablePath, byte[] miSearchValue) {
            try { darkages = File.ReadAllBytes(daExcutablePath); } catch(Exception e) { DisplayExceptionMessage(e); return false; }

            for(uint i = 0; i < darkages.Length - miSearchValue.Length; i++)
                for(byte ii = 0; ii < miSearchValue.Length; ii++) {
                    if(darkages[i + ii] != miSearchValue[ii]) break;
                    if(ii == miSearchValue.Length - 1) miOffsets.Add(i + (uint)miSearchValue.Length - 2);
                }
            return miOffsets.Count > 0;
        }
        private bool FindSetMIValuesRegEx(string daExcutablePath, string miSearchValue) {
            try { darkages = File.ReadAllBytes(daExcutablePath); } catch(Exception e) { DisplayExceptionMessage(e); return false; }

            MatchCollection mc = Regex.Matches(BitConverter.ToString(darkages), miSearchValue);
            foreach(Match A in mc) miOffsets.Add((uint)(A.Index - A.Index / 3) / 2 + 4);
            return miOffsets.Count > 0;
        }

        private void buttonSaveDarkagesMI_Click(object sender, EventArgs e) {
            byte replacmentByte;
            if(!byte.TryParse(textBoxReplacmentByte.Text, NumberStyles.HexNumber, CultureInfo.InvariantCulture, out replacmentByte)) { MessageBox.Show("Please Make Sure the Replacement Byte is Valid"); return; }
            if(listBoxMatches.SelectedIndex == -1) { MessageBox.Show("Please Make Sure an Offset in the Matches List Box is Selected"); return; }

            if(saveFileDialogDarkagesMI.ShowDialog(this) == DialogResult.OK) {
                darkages[(uint)listBoxMatches.SelectedValue] = replacmentByte;
                try { File.WriteAllBytes(saveFileDialogDarkagesMI.FileName, darkages); } catch(Exception e1) { DisplayExceptionMessage(e1); return; }
            }
        }

        private static void DisplayExceptionMessage(Exception e) { MessageBox.Show("eMessage: " + e.Message + Environment.NewLine + "eSource: " + e.Source + Environment.NewLine + "eStackTrace: " + e.StackTrace); }
    }
}